"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.query = exports.batchGet = exports.put = exports.update = exports.timestampCondition = exports.updateExpression = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _memoryCache = _interopRequireDefault(require("memory-cache"));

var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));

var _faults = require("./faults");

var _print = require("./print");

const updateExpression = Item => {
  const keys = Object.keys(Item);
  const ExpressionAttributeNames = keys.filter(attrName => Item[attrName] !== undefined).map(attrName => ({
    [`#${attrName}`]: attrName
  })).reduce(_merge.default, {});
  const ExpressionAttributeValues = keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => ({
    [`:${attrName}`]: Item[attrName]
  })).reduce(_merge.default, {});
  let UpdateExpression = `SET ${keys.filter(attrName => Item[attrName] !== undefined && Item[attrName] !== null).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`;
  const UpdateExpressionRemove = keys.filter(attrName => Item[attrName] === null).map(attrName => `#${attrName}`).join(', ');

  if (UpdateExpressionRemove.length) {
    UpdateExpression = `${UpdateExpression} REMOVE ${UpdateExpressionRemove}`;
  }

  return {
    ExpressionAttributeNames,
    ExpressionAttributeValues,
    UpdateExpression,
    ReturnValues: 'ALL_NEW'
  };
};

exports.updateExpression = updateExpression;

const timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});

exports.timestampCondition = timestampCondition;

const update = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME || process.env.EVENT_TABLE_NAME,
  updateRequestField = 'updateRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[updateRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.update(uow[updateRequestField]).then(updateResponse => ({ ...uow,
      updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.update = update;

const put = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  putRequestField = 'putRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[putRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.put(uow[putRequestField]).then(putResponse => ({ ...uow,
      putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.put = put;

const batchGet = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  batchGetRequestField = 'batchGetRequest',
  batchGetResponseField = 'batchGetResponse',
  parallel = Number(process.env.GET_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[batchGetRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const p = connector.batchGet(uow[batchGetRequestField]).then(batchGetResponse => ({ ...uow,
      [batchGetResponseField]: batchGetResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.batchGet = batchGet;

const query = (
/* istanbul ignore next */
{
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME || process.env.ENTITY_TABLE_NAME,
  queryRequestField = 'queryRequest',
  queryResponseField = 'queryResponse',
  parallel = Number(process.env.QUERY_PARALLEL) || Number(process.env.PARALLEL) || 4,
  timeout = Number(process.env.DYNAMODB_TIMEOUT) || Number(process.env.TIMEOUT) || 1000
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName,
    timeout
  });

  const invoke = uow => {
    if (!uow[queryRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const req = JSON.stringify(uow[queryRequestField]);

    const cached = _memoryCache.default.get(req);

    const p = (cached ? Promise.resolve(cached) : connector.query(uow[queryRequestField]).then(queryResponse => {
      _memoryCache.default.put(req, queryResponse);

      return queryResponse;
    })).then(queryResponse => ({ ...uow,
      [queryResponseField]: queryResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.query = query;